/*
 * File:   AQM1248A_PIC.c
 *
 * Created on December 7, 2021, 10:35 PM
 */

#include "xc.h"
#include "string.h"
#include "i2c1_PIC.h"
#include "AQM1248A_PIC.h"
#include "font_data.h"
#include "mcc_generated_files/spi1.h"

uint8_t v_buf[128][6];

void Init_LCD()
{
    LCD_CS =  LOW;
    LCD_RS =  LOW;

    SPI1_Exchange8bit(0xAE);
    SPI1_Exchange8bit(0xA0);
    SPI1_Exchange8bit(0xC8);
    SPI1_Exchange8bit(0xA3);

    SPI1_Exchange8bit(0x2C);
    __delay_ms(50);
    SPI1_Exchange8bit(0x2E);
    __delay_ms(50);

    SPI1_Exchange8bit(0x2F);

    SPI1_Exchange8bit(0x23);
    SPI1_Exchange8bit(0x81);
    SPI1_Exchange8bit(0x1C);

    SPI1_Exchange8bit(0xA4);
    SPI1_Exchange8bit(0x40);
    SPI1_Exchange8bit(0xA6);
    SPI1_Exchange8bit(0xAF);

   LCD_CS = HIGH;
}

void LCD_CLS(uint8_t data)
{
    uint8_t a,b;

    LCD_CS = LOW;
    for(b = 0; b < 6; b ++)
    {
        LCD_RS = LOW;
        SPI1_Exchange8bit(0xB0 + b);
        SPI1_Exchange8bit(0x10);
        SPI1_Exchange8bit(0x00);

        LCD_RS = HIGH;
        for(a = 0; a < 128; a ++)
        {
            SPI1_Exchange8bit(data);
            v_buf[a][b] = data;
        }
    }
    LCD_CS = HIGH;
}

//----------------------------------------------------
//  _̕`
//  int x_data  X positon   0 -> 128
//  int x_data  Y positon   0 -> 48
//  int cl      color 0: white  1:black
//----------------------------------------------------
void LCD_PSET(int x_data, int y_data, int cl)
{
    uint8_t a,b,c;

//  y_data
    a = y_data >> 3; b = y_data & 0x07;
    c = 0x1;
    while(b) { c <<= 1; b --; }

    if(cl) v_buf[x_data][a] |= c;
    else { c = ~c; v_buf[x_data][a] &= c; }

    LCD_CS = LOW;
    LCD_RS = LOW;
    SPI1_Exchange8bit(0xB0 + a);
    c = x_data >> 4; c |= 0x10;
    SPI1_Exchange8bit(c);
    c = x_data & 0xf;
    SPI1_Exchange8bit(c);
   
    LCD_RS = HIGH;
    SPI1_Exchange8bit(v_buf[x_data][a]);
    
    LCD_CS = HIGH;
}

//----------------------------------------------------
//  Font̕`
//  int x_data  X positon   0 -> 128
//  int y_data  Y positon   0 -> 48
//  char c_data Data
//  int cl      color 0: white  1:black
//----------------------------------------------------
void LCD_Print_C(int x_data, int y_data, char c_data, int cl)
{
    int a,b,c,d;
    unsigned char s,data_c;
    
    data_c = c_data;
    if(data_c > 0xbf) data_c -= 0x40;
    a = data_c - 0x20;
    for(b = 0; b < 5; b ++)
    {
        s = 0x1;
        for(c = 0; c < 8; c ++)
        {
            d = 0;
            if(Font[a][b] & s) d=1;
            if(cl == 0) d = !d;
            LCD_PSET(x_data,y_data + c,d);
            s <<= 1;
        }
        x_data ++;
    }
    for(c = 0; c < 8; c ++)
    {
        d = 0;
        if(cl == 0) d = 1;
        LCD_PSET(x_data,y_data + c,d);
    }
}

//----------------------------------------------------
//  Str̕`
//  int x_data  X positon   0 -> 128
//  int y_data  Y positon   0 -> 48
//  char *c_data Data
//  int cl      color 0: white  1:black
//----------------------------------------------------
void LCD_Print_Str(int x_data, int y_data, char * c_data, int cl)
{
    int a;

    a = strlen((char *)c_data);
    while(a)
    {
        if(*c_data == 0xff) 
            { c_data ++; a --; }
        
        LCD_Print_C(x_data,y_data,*c_data,cl);
        a --; x_data += 6; c_data ++;
    }
}

//----------------------------------------------------
//   `֐
//
//  int x0      start x
//  int y0      start y
//  int x1      end x
//  int y1      end y
//  int cl      color 0: white  1:black
//----------------------------------------------------
#define abs(a)  (((a)>0) ? (a) : -(a))
void LCD_LINE(int x0, int y0, int x1, int y1, int cl)
{
    int steep, t;
    int deltax, deltay, error;
    int x, y;
    int ystep;

    /// ̑傫ق߂
    steep = (abs(y1 - y0) > abs(x1 - x0));
    /// A̓ւ
    if(steep)
    {
        t = x0; x0 = y0; y0 = t;
        t = x1; x1 = y1; y1 = t;
    }
    if(x0 > x1)
    {
        t = x0; x0 = x1; x1 = t;
        t = y0; y0 = y1; y1 = t;
    }
    deltax = x1 - x0;                       //  XvZ
    deltay = abs(y1 - y0);
    error = 0;
    y = y0;
    /// XŃXebv̐؂ւ
    if(y0 < y1) ystep = 1; else ystep = -1;
    /// _ŕ`
    for(x = x0; x < x1 + 1; x++)
    {
        if(steep) LCD_PSET(y,x,cl); else LCD_PSET(x,y,cl);
        error += deltay;
        if((error << 1) >= deltax)
        {
            y += ystep;
            error -= deltax;
        }
    }
}